import {Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges} from '@angular/core';
import {animate, state, style, transition, trigger} from '@angular/animations';
import {DataSource} from '@angular/cdk/table';
import {Observable, of} from 'rxjs';
import {TabuMiddlewareService} from '../dao/TabuMiddlewareService';
import {ViewModus} from '../dao/viewModus';

export interface CardResultHistory {
  answer: Answer;
  solution: string;
  redTurn: number;
  tabu1: string;
  tabu2: string;
  tabu3: string;
  tabu4: string;
  tabu5: string;
  s2cID: number;
}

export enum Answer {
  zeitLaueft,
  zeitAbgelaufen,
  richtig,
  uebersprungen,
  tabooh,
  KeineEingabe
}

@Component({
  selector: 'app-round-history',
  templateUrl: './round-history.component.html',
  styleUrls: ['./round-history.component.scss'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({ height: '0px', minHeight: '0', visibility: 'hidden' })),
      state('expanded', style({ height: '*', visibility: 'visible' })),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],
})
export class RoundHistoryComponent extends DataSource<any> implements OnInit, OnChanges {
  @Input()
  round: number | undefined;
  @Input()
  history: any | undefined;
  @Input()
  sessionName: string | undefined;
  @Input()
  canEdit: boolean | undefined;
  @Output()
  gameStatus = new EventEmitter();
  @Output()
  canUpdate = new EventEmitter();
  view = 'desktopView';
  redTurn = '';
  lastS2CID = 0;

  elementRowStyle = 'element-row';
  roundHistory: CardResultHistory[] = [];
  displayedColumns = ['answer', 'solution', 'tabu1', 'tabu2', 'tabu3', 'tabu4', 'tabu5'];


  // displayedColumns = ['position', 'name', 'weight'];
  dataSource = this.connect();
  answer = Answer;

  expandedElement: any;
  isExpansionDetailRow = (i: number, row: object) => row.hasOwnProperty('detailRow');

  constructor(
    private service: TabuMiddlewareService,
    private screen: ViewModus) {
    super();
    screen.$view.subscribe(
      view => {
        this.view = view;
        this.setDisplayColumns();
      });
  }

  ngOnInit(): void {
    this.view = this.screen.actuallView;
    this.setDisplayColumns();
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.hasOwnProperty('round')) {
      this.round = changes.round.currentValue;
    }
    if (changes.hasOwnProperty('history')) {
      this.history = changes.history.currentValue;
      this.getRoundDataFromHistory();
    }
    if (changes.hasOwnProperty('sessionName')) {
      this.sessionName = changes.sessionName.currentValue;
    }
    if (changes.hasOwnProperty('canEdit')) {
      this.canEdit = changes.canEdit.currentValue;
      if (this.canEdit) {
        this.elementRowStyle = 'element-row';
      } else {
        this.elementRowStyle = 'element-row-noEdit';
      }
    }
  }

  setDisplayColumns(): void {
    if (this.view === 'mobileView') {
      this.displayedColumns = ['answer', 'solution'];
    } else {
      this.displayedColumns = ['answer', 'solution', 'tabu1', 'tabu2', 'tabu3', 'tabu4', 'tabu5'];
    }
  }

  getRoundDataFromHistory(): void {
    this.roundHistory = [];
    let redTurn = null;
    for (const h of this.history) {
      if (h.Round === this.round) {
        let cardResultName = Answer.KeineEingabe;
        const answer = JSON.parse(h.CardResultID);
        if (answer === 0) {
          cardResultName = Answer.zeitLaueft;
        } else if (answer === 1) {
          cardResultName = Answer.zeitAbgelaufen;
        } else if (answer === 2) {
          cardResultName = Answer.richtig;
        } else if (answer === 3) {
          cardResultName = Answer.uebersprungen;
        } else if (answer === 4) {
          cardResultName = Answer.tabooh;
        }
        const newCardResultHistory: CardResultHistory = {
          s2cID: JSON.parse(h.S2CID),
          solution: h.Solution,
          redTurn: h.RedTurn,
          answer: cardResultName,
          tabu1: h.Tabu1,
          tabu2: h.Tabu2,
          tabu3: h.Tabu3,
          tabu4: h.Tabu4,
          tabu5: h.Tabu5
        };
        this.roundHistory.push(newCardResultHistory);
        // First Found
        if (redTurn === null) {
          this.lastS2CID = JSON.parse(h.S2CID);
          redTurn = JSON.parse(h.RedTurn);
          if (redTurn === 1) {
            this.redTurn = 'red';
          } else if (redTurn === 0) {
            this.redTurn = 'blue';
          }
        }
      }
    }
    this.disconnect();
    this.dataSource = this.connect();
    /*if (this.getExpansionDetailObjekt !== null) {
      this.isExpansionDetailRow(this.getExpansionDetailNumber, this.getExpansionDetailObjekt);
    }*/
  }

  updateAnswer(element: CardResultHistory, answer: Answer): void {
    const s2cCard = this.roundHistory.findIndex(x => x.s2cID === element.s2cID);
    const oldAnswer = this.roundHistory[s2cCard].answer;
    this.roundHistory[s2cCard].answer = answer;
    let cardResultID = 0;
    let point = 0;
    // Richtig Button selected
    if (answer === Answer.richtig) {
      cardResultID = 2;
      if (oldAnswer === Answer.tabooh) {
        point = 2;
      } else if (oldAnswer === Answer.uebersprungen || oldAnswer === Answer.zeitAbgelaufen) {
        point = 1;
      }
    }
    // Ueberspringen or Zeit abgelaufen Button selected
    if (answer === Answer.uebersprungen || answer === Answer.zeitAbgelaufen) {
      if (answer === Answer.zeitAbgelaufen) {
        cardResultID = 1;
      }
      if (answer === Answer.uebersprungen) {
        cardResultID = 3;
      }
      if (oldAnswer === Answer.tabooh) {
        point = 1;
      } else if (oldAnswer === Answer.richtig) {
        point = -1;
      }
    }
    // Tabooh Button selected
    if (answer === Answer.tabooh) {
      cardResultID = 4;
      if (oldAnswer === Answer.richtig) {
        point = -2;
      } else if (oldAnswer === Answer.zeitAbgelaufen || oldAnswer === Answer.uebersprungen) {
        point = -1;
      }
    }
    const timestamp = Date.now();
    this.canUpdate.emit(false);
    this.service.historyChangePoint(
      {
        spielname: this.sessionName,
        redTurn: element.redTurn,
        point,
        s2cID: element.s2cID,
        cardResultID,
        time: timestamp
      }
    ).then(value => {
      const status = JSON.parse(value.status);
      if (status) {
        this.canUpdate.emit(true);
        this.gameStatus.emit(value);
      }
    }).catch(reason => console.log(reason)
    );
  }

  /** Connect function called by the table to retrieve one stream containing the data to render. */
  connect(): Observable<CardResultHistory[]> {
    const rows: any = [];
    this.roundHistory.forEach(element => rows.push(element, { detailRow: true, element }));
    return of(rows);
  }

  disconnect(): void { }
}
